@@ -1,27 +1,27 @@
 # frozen_string_literal: true
 
-require 'sidekiq/version'
-fail "Sidekiq #{Sidekiq::VERSION} does not support Ruby versions below 2.2.2." if RUBY_PLATFORM != 'java' && Gem::Version.new(RUBY_VERSION) < Gem::Version.new('2.2.2')
+require "sidekiq/version"
+fail "Sidekiq #{Sidekiq::VERSION} does not support Ruby versions below 2.5.0." if RUBY_PLATFORM != "java" && Gem::Version.new(RUBY_VERSION) < Gem::Version.new("2.5.0")
 
-require 'sidekiq/logging'
-require 'sidekiq/client'
-require 'sidekiq/worker'
-require 'sidekiq/redis_connection'
-require 'sidekiq/delay'
+require "sidekiq/logger"
+require "sidekiq/client"
+require "sidekiq/worker"
+require "sidekiq/redis_connection"
+require "sidekiq/delay"
 
-require 'json'
+require "json"
 
 module Sidekiq
-  NAME = 'Sidekiq'
-  LICENSE = 'See LICENSE and the LGPL-3.0 for licensing details.'
+  NAME = "Sidekiq"
+  LICENSE = "See LICENSE and the LGPL-3.0 for licensing details."
 
   DEFAULTS = {
     queues: [],
     labels: [],
     concurrency: 10,
-    require: '.',
+    require: ".",
     environment: nil,
-    timeout: 8,
+    timeout: 25,
     poll_interval_average: nil,
     average_scheduled_poll_interval: 5,
     error_handlers: [],
@@ -38,8 +38,8 @@ module Sidekiq
   }
 
   DEFAULT_WORKER_OPTIONS = {
-    'retry' => true,
-    'queue' => 'default'
+    "retry" => true,
+    "queue" => "default",
   }
 
   FAKE_INFO = {
@@ -47,7 +47,7 @@ module Sidekiq
     "uptime_in_days" => "9999",
     "connected_clients" => "9999",
     "used_memory_human" => "9P",
-    "used_memory_peak_human" => "9P"
+    "used_memory_peak_human" => "9P",
   }
 
   def self.❨╯°□°❩╯︵┻━┻
@@ -96,9 +96,13 @@ module Sidekiq
       begin
         yield conn
       rescue Redis::CommandError => ex
-        #2550 Failover can cause the server to become a replica, need
+        # 2550 Failover can cause the server to become a replica, need
         # to disconnect and reopen the socket to get back to the primary.
-        (conn.disconnect!; retryable = false; retry) if retryable && ex.message =~ /READONLY/
+        if retryable && ex.message =~ /READONLY/
+          conn.disconnect!
+          retryable = false
+          retry
+        end
         raise
       end
     end
@@ -106,19 +110,17 @@ module Sidekiq
 
   def self.redis_info
     redis do |conn|
-      begin
-        # admin commands can't go through redis-namespace starting
-        # in redis-namespace 2.0
-        if conn.respond_to?(:namespace)
-          conn.redis.info
-        else
-          conn.info
-        end
-      rescue Redis::CommandError => ex
-        #2850 return fake version when INFO command has (probably) been renamed
-        raise unless ex.message =~ /unknown command/
-        FAKE_INFO
+      # admin commands can't go through redis-namespace starting
+      # in redis-namespace 2.0
+      if conn.respond_to?(:namespace)
+        conn.redis.info
+      else
+        conn.info
       end
+    rescue Redis::CommandError => ex
+      # 2850 return fake version when INFO command has (probably) been renamed
+      raise unless /unknown command/.match?(ex.message)
+      FAKE_INFO
     end
   end
 
@@ -152,16 +154,11 @@ module Sidekiq
 
   def self.default_worker_options=(hash)
     # stringify
-    @default_worker_options = default_worker_options.merge(Hash[hash.map{|k, v| [k.to_s, v]}])
-  end
-  def self.default_worker_options
-    defined?(@default_worker_options) ? @default_worker_options : DEFAULT_WORKER_OPTIONS
+    @default_worker_options = default_worker_options.merge(Hash[hash.map { |k, v| [k.to_s, v] }])
   end
 
-  def self.default_retries_exhausted=(prok)
-    logger.info { "default_retries_exhausted is deprecated, please use `config.death_handlers << -> {|job, ex| }`" }
-    return nil unless prok
-    death_handlers << prok
+  def self.default_worker_options
+    defined?(@default_worker_options) ? @default_worker_options : DEFAULT_WORKER_OPTIONS
   end
 
   ##
@@ -180,15 +177,41 @@ module Sidekiq
   def self.load_json(string)
     JSON.parse(string)
   end
+
   def self.dump_json(object)
     JSON.generate(object)
   end
 
-  def self.logger
-    Sidekiq::Logging.logger
+  def self.log_formatter
+    @log_formatter ||= if ENV["DYNO"]
+      Sidekiq::Logger::Formatters::WithoutTimestamp.new
+    else
+      Sidekiq::Logger::Formatters::Pretty.new
+    end
   end
-  def self.logger=(log)
-    Sidekiq::Logging.logger = log
+
+  def self.log_formatter=(log_formatter)
+    @log_formatter = log_formatter
+    logger.formatter = log_formatter
+  end
+
+  def self.logger
+    @logger ||= Sidekiq::Logger.new(STDOUT, level: Logger::INFO)
+  end
+
+  def self.logger=(logger)
+    if logger.nil?
+      self.logger.level = Logger::FATAL
+      return self.logger
+    end
+
+    logger.extend(Sidekiq::LoggingUtils)
+
+    @logger = logger
+  end
+
+  def self.pro?
+    defined?(Sidekiq::Pro)
   end
 
   # How frequently Redis should be checked by a random Sidekiq process for
@@ -197,7 +220,7 @@ module Sidekiq
   #
   # See sidekiq/scheduled.rb for an in-depth explanation of this value
   def self.average_scheduled_poll_interval=(interval)
-    self.options[:average_scheduled_poll_interval] = interval
+    options[:average_scheduled_poll_interval] = interval
   end
 
   # Register a proc to handle any error which occurs within the Sidekiq process.
@@ -208,7 +231,7 @@ module Sidekiq
   #
   # The default error handler logs errors to Sidekiq.logger.
   def self.error_handlers
-    self.options[:error_handlers]
+    options[:error_handlers]
   end
 
   # Register a block to run at a point in the Sidekiq lifecycle.
@@ -234,4 +257,4 @@ module Sidekiq
   class Shutdown < Interrupt; end
 end
 
-require 'sidekiq/rails' if defined?(::Rails::Engine)
+require "sidekiq/rails" if defined?(::Rails::Engine)
