@@ -1,24 +1,12 @@
 # frozen_string_literal: true
-require 'sidekiq'
+
+require "sidekiq"
+
+require "zlib"
+require "base64"
 
 module Sidekiq
-
-  module RedisScanner
-    def sscan(conn, key)
-      cursor = '0'
-      result = []
-      loop do
-        cursor, values = conn.sscan(key, cursor)
-        result.push(*values)
-        break if cursor == '0'
-      end
-      result
-    end
-  end
-
   class Stats
-    include RedisScanner
-
     def initialize
       fetch_stats!
     end
@@ -64,61 +52,65 @@ module Sidekiq
     end
 
     def fetch_stats!
-      pipe1_res = Sidekiq.redis do |conn|
+      pipe1_res = Sidekiq.redis { |conn|
         conn.pipelined do
-          conn.get('stat:processed')
-          conn.get('stat:failed')
-          conn.zcard('schedule')
-          conn.zcard('retry')
-          conn.zcard('dead')
-          conn.scard('processes')
-          conn.lrange('queue:default', -1, -1)
+          conn.get("stat:processed")
+          conn.get("stat:failed")
+          conn.zcard("schedule")
+          conn.zcard("retry")
+          conn.zcard("dead")
+          conn.scard("processes")
+          conn.lrange("queue:default", -1, -1)
         end
-      end
+      }
 
-      processes = Sidekiq.redis do |conn|
-        sscan(conn, 'processes')
-      end
+      processes = Sidekiq.redis { |conn|
+        conn.sscan_each("processes").to_a
+      }
 
-      queues = Sidekiq.redis do |conn|
-        sscan(conn, 'queues')
-      end
+      queues = Sidekiq.redis { |conn|
+        conn.sscan_each("queues").to_a
+      }
 
-      pipe2_res = Sidekiq.redis do |conn|
+      pipe2_res = Sidekiq.redis { |conn|
         conn.pipelined do
-          processes.each {|key| conn.hget(key, 'busy') }
-          queues.each {|queue| conn.llen("queue:#{queue}") }
+          processes.each { |key| conn.hget(key, "busy") }
+          queues.each { |queue| conn.llen("queue:#{queue}") }
         end
-      end
+      }
 
       s = processes.size
-      workers_size = pipe2_res[0...s].map(&:to_i).inject(0, &:+)
-      enqueued     = pipe2_res[s..-1].map(&:to_i).inject(0, &:+)
+      workers_size = pipe2_res[0...s].sum(&:to_i)
+      enqueued = pipe2_res[s..-1].sum(&:to_i)
 
       default_queue_latency = if (entry = pipe1_res[6].first)
-                                job = Sidekiq.load_json(entry) rescue {}
-                                now = Time.now.to_f
-                                thence = job['enqueued_at'] || now
-                                now - thence
-                              else
-                                0
-                              end
+        job = begin
+                Sidekiq.load_json(entry)
+              rescue
+                {}
+              end
+        now = Time.now.to_f
+        thence = job["enqueued_at"] || now
+        now - thence
+      else
+        0
+      end
       @stats = {
-        processed:             pipe1_res[0].to_i,
-        failed:                pipe1_res[1].to_i,
-        scheduled_size:        pipe1_res[2],
-        retry_size:            pipe1_res[3],
-        dead_size:             pipe1_res[4],
-        processes_size:        pipe1_res[5],
+        processed: pipe1_res[0].to_i,
+        failed: pipe1_res[1].to_i,
+        scheduled_size: pipe1_res[2],
+        retry_size: pipe1_res[3],
+        dead_size: pipe1_res[4],
+        processes_size: pipe1_res[5],
 
         default_queue_latency: default_queue_latency,
-        workers_size:          workers_size,
-        enqueued:              enqueued
+        workers_size: workers_size,
+        enqueued: enqueued,
       }
     end
 
     def reset(*stats)
-      all   = %w(failed processed)
+      all = %w[failed processed]
       stats = stats.empty? ? all : all & stats.flatten.compact.map(&:to_s)
 
       mset_args = []
@@ -138,26 +130,18 @@ module Sidekiq
     end
 
     class Queues
-      include RedisScanner
-
       def lengths
         Sidekiq.redis do |conn|
-          queues = sscan(conn, 'queues')
+          queues = conn.sscan_each("queues").to_a
 
-          lengths = conn.pipelined do
+          lengths = conn.pipelined {
             queues.each do |queue|
               conn.llen("queue:#{queue}")
             end
-          end
+          }
 
-          i = 0
-          array_of_arrays = queues.inject({}) do |memo, queue|
-            memo[queue] = lengths[i]
-            i += 1
-            memo
-          end.sort_by { |_, size| size }
-
-          Hash[array_of_arrays.reverse]
+          array_of_arrays = queues.zip(lengths).sort_by { |_, size| -size }
+          Hash[array_of_arrays]
         end
       end
     end
@@ -179,18 +163,12 @@ module Sidekiq
       private
 
       def date_stat_hash(stat)
-        i = 0
         stat_hash = {}
-        keys = []
-        dates = []
+        dates = @start_date.downto(@start_date - @days_previous + 1).map { |date|
+          date.strftime("%Y-%m-%d")
+        }
 
-        while i < @days_previous
-          date = @start_date - i
-          datestr = date.strftime("%Y-%m-%d")
-          keys << "stat:#{stat}:#{datestr}"
-          dates << datestr
-          i += 1
-        end
+        keys = dates.map { |datestr| "stat:#{stat}:#{datestr}" }
 
         begin
           Sidekiq.redis do |conn|
@@ -222,18 +200,17 @@ module Sidekiq
   #
   class Queue
     include Enumerable
-    extend RedisScanner
 
     ##
     # Return all known queues within Redis.
     #
     def self.all
-      Sidekiq.redis { |c| sscan(c, 'queues') }.sort.map { |q| Sidekiq::Queue.new(q) }
+      Sidekiq.redis { |c| c.sscan_each("queues").to_a }.sort.map { |q| Sidekiq::Queue.new(q) }
     end
 
     attr_reader :name
 
-    def initialize(name="default")
+    def initialize(name = "default")
       @name = name.to_s
       @rname = "queue:#{name}"
     end
@@ -253,13 +230,13 @@ module Sidekiq
     #
     # @return Float
     def latency
-      entry = Sidekiq.redis do |conn|
+      entry = Sidekiq.redis { |conn|
         conn.lrange(@rname, -1, -1)
-      end.first
+      }.first
       return 0 unless entry
       job = Sidekiq.load_json(entry)
       now = Time.now.to_f
-      thence = job['enqueued_at'] || now
+      thence = job["enqueued_at"] || now
       now - thence
     end
 
@@ -269,12 +246,12 @@ module Sidekiq
       page = 0
       page_size = 50
 
-      while true do
+      loop do
         range_start = page * page_size - deleted_size
-        range_end   = range_start + page_size - 1
-        entries = Sidekiq.redis do |conn|
+        range_end = range_start + page_size - 1
+        entries = Sidekiq.redis { |conn|
           conn.lrange @rname, range_start, range_end
-        end
+        }
         break if entries.empty?
         page += 1
         entries.each do |entry|
@@ -315,11 +292,11 @@ module Sidekiq
     attr_reader :item
     attr_reader :value
 
-    def initialize(item, queue_name=nil)
+    def initialize(item, queue_name = nil)
       @args = nil
       @value = item
       @item = item.is_a?(Hash) ? item : parse(item)
-      @queue = queue_name || @item['queue']
+      @queue = queue_name || @item["queue"]
     end
 
     def parse(item)
@@ -334,7 +311,7 @@ module Sidekiq
     end
 
     def klass
-      self['class']
+      self["class"]
     end
 
     def display_class
@@ -345,16 +322,16 @@ module Sidekiq
                      "#{target}.#{method}"
                    end
                  when "ActiveJob::QueueAdapters::SidekiqAdapter::JobWrapper"
-                   job_class = @item['wrapped'] || args[0]
-                   if 'ActionMailer::DeliveryJob' == job_class
+                   job_class = @item["wrapped"] || args[0]
+                   if job_class == "ActionMailer::DeliveryJob" || job_class == "ActionMailer::MailDeliveryJob"
                      # MailerClass#mailer_method
-                     args[0]['arguments'][0..1].join('#')
+                     args[0]["arguments"][0..1].join("#")
                    else
-                    job_class
+                     job_class
                    end
                  else
                    klass
-                 end
+      end
     end
 
     def display_args
@@ -365,53 +342,68 @@ module Sidekiq
                     arg
                   end
                 when "ActiveJob::QueueAdapters::SidekiqAdapter::JobWrapper"
-                  job_args = self['wrapped'] ? args[0]["arguments"] : []
-                  if 'ActionMailer::DeliveryJob' == (self['wrapped'] || args[0])
+                  job_args = self["wrapped"] ? args[0]["arguments"] : []
+                  if (self["wrapped"] || args[0]) == "ActionMailer::DeliveryJob"
                     # remove MailerClass, mailer_method and 'deliver_now'
                     job_args.drop(3)
+                  elsif (self["wrapped"] || args[0]) == "ActionMailer::MailDeliveryJob"
+                    # remove MailerClass, mailer_method and 'deliver_now'
+                    job_args.drop(3).first["args"]
                   else
                     job_args
                   end
                 else
-                  if self['encrypt']
+                  if self["encrypt"]
                     # no point in showing 150+ bytes of random garbage
-                    args[-1] = '[encrypted data]'
+                    args[-1] = "[encrypted data]"
                   end
                   args
-                end
+      end
     end
 
     def args
-      @args || @item['args']
+      @args || @item["args"]
     end
 
     def jid
-      self['jid']
+      self["jid"]
     end
 
     def enqueued_at
-      self['enqueued_at'] ? Time.at(self['enqueued_at']).utc : nil
+      self["enqueued_at"] ? Time.at(self["enqueued_at"]).utc : nil
     end
 
     def created_at
-      Time.at(self['created_at'] || self['enqueued_at'] || 0).utc
+      Time.at(self["created_at"] || self["enqueued_at"] || 0).utc
     end
 
-    def queue
-      @queue
+    def tags
+      self["tags"] || []
     end
 
+    def error_backtrace
+      # Cache nil values
+      if defined?(@error_backtrace)
+        @error_backtrace
+      else
+        value = self["error_backtrace"]
+        @error_backtrace = value && uncompress_backtrace(value)
+      end
+    end
+
+    attr_reader :queue
+
     def latency
       now = Time.now.to_f
-      now - (@item['enqueued_at'] || @item['created_at'] || now)
+      now - (@item["enqueued_at"] || @item["created_at"] || now)
     end
 
     ##
     # Remove this job from the queue.
     def delete
-      count = Sidekiq.redis do |conn|
+      count = Sidekiq.redis { |conn|
         conn.lrem("queue:#{@queue}", 1, @value)
-      end
+      }
       count != 0
     end
 
@@ -425,13 +417,28 @@ module Sidekiq
     private
 
     def safe_load(content, default)
-      begin
-        yield(*YAML.load(content))
-      rescue => ex
-        # #1761 in dev mode, it's possible to have jobs enqueued which haven't been loaded into
-        # memory yet so the YAML can't be loaded.
-        Sidekiq.logger.warn "Unable to load YAML: #{ex.message}" unless Sidekiq.options[:environment] == 'development'
-        default
+      yield(*YAML.load(content))
+    rescue => ex
+      # #1761 in dev mode, it's possible to have jobs enqueued which haven't been loaded into
+      # memory yet so the YAML can't be loaded.
+      Sidekiq.logger.warn "Unable to load YAML: #{ex.message}" unless Sidekiq.options[:environment] == "development"
+      default
+    end
+
+    def uncompress_backtrace(backtrace)
+      if backtrace.is_a?(Array)
+        # Handle old jobs with raw Array backtrace format
+        backtrace
+      else
+        decoded = Base64.decode64(backtrace)
+        uncompressed = Zlib::Inflate.inflate(decoded)
+        begin
+          Sidekiq.load_json(uncompressed)
+        rescue
+          # Handle old jobs with marshalled backtrace format
+          # TODO Remove in 7.x
+          Marshal.load(uncompressed)
+        end
       end
     end
   end
@@ -459,8 +466,9 @@ module Sidekiq
     end
 
     def reschedule(at)
-      delete
-      @parent.schedule(at, item)
+      Sidekiq.redis do |conn|
+        conn.zincrby(@parent.name, at.to_f - @score, Sidekiq.dump_json(@item))
+      end
     end
 
     def add_to_queue
@@ -473,7 +481,7 @@ module Sidekiq
     def retry
       remove_job do |message|
         msg = Sidekiq.load_json(message)
-        msg['retry_count'] -= 1 if msg['retry_count']
+        msg["retry_count"] -= 1 if msg["retry_count"]
         Sidekiq::Client.push(msg)
       end
     end
@@ -487,45 +495,44 @@ module Sidekiq
     end
 
     def error?
-      !!item['error_class']
+      !!item["error_class"]
     end
 
     private
 
     def remove_job
       Sidekiq.redis do |conn|
-        results = conn.multi do
+        results = conn.multi {
           conn.zrangebyscore(parent.name, score, score)
           conn.zremrangebyscore(parent.name, score, score)
-        end.first
+        }.first
 
         if results.size == 1
           yield results.first
         else
           # multiple jobs with the same score
           # find the one with the right JID and push it
-          hash = results.group_by do |message|
+          matched, nonmatched = results.partition { |message|
             if message.index(jid)
               msg = Sidekiq.load_json(message)
-              msg['jid'] == jid
+              msg["jid"] == jid
             else
               false
             end
-          end
+          }
 
-          msg = hash.fetch(true, []).first
+          msg = matched.first
           yield msg if msg
 
           # push the rest back onto the sorted set
           conn.multi do
-            hash.fetch(false, []).each do |message|
+            nonmatched.each do |message|
               conn.zadd(parent.name, score.to_f.to_s, message)
             end
           end
         end
       end
     end
-
   end
 
   class SortedSet
@@ -542,6 +549,17 @@ module Sidekiq
       Sidekiq.redis { |c| c.zcard(name) }
     end
 
+    def scan(match, count = 100)
+      return to_enum(:scan, match, count) unless block_given?
+
+      match = "*#{match}*" unless match.include?("*")
+      Sidekiq.redis do |conn|
+        conn.zscan_each(name, match: match, count: count) do |entry, score|
+          yield SortedEntry.new(self, score, entry)
+        end
+      end
+    end
+
     def clear
       Sidekiq.redis do |conn|
         conn.del(name)
@@ -551,7 +569,6 @@ module Sidekiq
   end
 
   class JobSet < SortedSet
-
     def schedule(timestamp, message)
       Sidekiq.redis do |conn|
         conn.zadd(name, timestamp.to_f.to_s, Sidekiq.dump_json(message))
@@ -564,44 +581,55 @@ module Sidekiq
       page = -1
       page_size = 50
 
-      while true do
+      loop do
         range_start = page * page_size + offset_size
-        range_end   = range_start + page_size - 1
-        elements = Sidekiq.redis do |conn|
+        range_end = range_start + page_size - 1
+        elements = Sidekiq.redis { |conn|
           conn.zrange name, range_start, range_end, with_scores: true
-        end
+        }
         break if elements.empty?
         page -= 1
-        elements.reverse.each do |element, score|
+        elements.reverse_each do |element, score|
           yield SortedEntry.new(self, score, element)
         end
         offset_size = initial_size - @_size
       end
     end
 
+    ##
+    # Fetch jobs that match a given time or Range. Job ID is an
+    # optional second argument.
     def fetch(score, jid = nil)
-      elements = Sidekiq.redis do |conn|
-        conn.zrangebyscore(name, score, score)
-      end
-
-      elements.inject([]) do |result, element|
-        entry = SortedEntry.new(self, score, element)
-        if jid
-          result << entry if entry.jid == jid
+      begin_score, end_score =
+        if score.is_a?(Range)
+          [score.first, score.last]
         else
-          result << entry
+          [score, score]
         end
-        result
+
+      elements = Sidekiq.redis { |conn|
+        conn.zrangebyscore(name, begin_score, end_score, with_scores: true)
+      }
+
+      elements.each_with_object([]) do |element, result|
+        data, job_score = element
+        entry = SortedEntry.new(self, job_score, data)
+        result << entry if jid.nil? || entry.jid == jid
       end
     end
 
     ##
     # Find the job with the given JID within this sorted set.
-    #
-    # This is a slow, inefficient operation.  Do not use under
-    # normal conditions.  Sidekiq Pro contains a faster version.
+    # This is a slower O(n) operation.  Do not use for app logic.
     def find_job(jid)
-      self.detect { |j| j.jid == jid }
+      Sidekiq.redis do |conn|
+        conn.zscan_each(name, match: "*#{jid}*", count: 100) do |entry, score|
+          job = JSON.parse(entry)
+          matched = job["jid"] == jid
+          return SortedEntry.new(self, score, entry) if matched
+        end
+      end
+      nil
     end
 
     def delete_by_value(name, value)
@@ -616,13 +644,14 @@ module Sidekiq
       Sidekiq.redis do |conn|
         elements = conn.zrangebyscore(name, score, score)
         elements.each do |element|
-          message = Sidekiq.load_json(element)
-          if message["jid"] == jid
-            ret = conn.zrem(name, element)
-            @_size -= 1 if ret
-            break ret
+          if element.index(jid)
+            message = Sidekiq.load_json(element)
+            if message["jid"] == jid
+              ret = conn.zrem(name, element)
+              @_size -= 1 if ret
+              break ret
+            end
           end
-          false
         end
       end
     end
@@ -644,7 +673,7 @@ module Sidekiq
   #   end.map(&:delete)
   class ScheduledSet < JobSet
     def initialize
-      super 'schedule'
+      super "schedule"
     end
   end
 
@@ -662,19 +691,15 @@ module Sidekiq
   #   end.map(&:delete)
   class RetrySet < JobSet
     def initialize
-      super 'retry'
+      super "retry"
     end
 
     def retry_all
-      while size > 0
-        each(&:retry)
-      end
+      each(&:retry) while size > 0
     end
 
     def kill_all
-      while size > 0
-        each(&:kill)
-      end
+      each(&:kill) while size > 0
     end
   end
 
@@ -683,15 +708,15 @@ module Sidekiq
   #
   class DeadSet < JobSet
     def initialize
-      super 'dead'
+      super "dead"
     end
 
-    def kill(message, opts={})
+    def kill(message, opts = {})
       now = Time.now.to_f
       Sidekiq.redis do |conn|
         conn.multi do
           conn.zadd(name, now.to_s, message)
-          conn.zremrangebyscore(name, '-inf', now - self.class.timeout)
+          conn.zremrangebyscore(name, "-inf", now - self.class.timeout)
           conn.zremrangebyrank(name, 0, - self.class.max_jobs)
         end
       end
@@ -708,9 +733,7 @@ module Sidekiq
     end
 
     def retry_all
-      while size > 0
-        each(&:retry)
-      end
+      each(&:retry) while size > 0
     end
 
     def self.max_jobs
@@ -724,16 +747,15 @@ module Sidekiq
 
   ##
   # Enumerates the set of Sidekiq processes which are actively working
-  # right now.  Each process send a heartbeat to Redis every 5 seconds
+  # right now.  Each process sends a heartbeat to Redis every 5 seconds
   # so this set should be relatively accurate, barring network partitions.
   #
   # Yields a Sidekiq::Process.
   #
   class ProcessSet
     include Enumerable
-    include RedisScanner
 
-    def initialize(clean_plz=true)
+    def initialize(clean_plz = true)
       cleanup if clean_plz
     end
 
@@ -742,50 +764,47 @@ module Sidekiq
     def cleanup
       count = 0
       Sidekiq.redis do |conn|
-        procs = sscan(conn, 'processes').sort
-        heartbeats = conn.pipelined do
+        procs = conn.sscan_each("processes").to_a.sort
+        heartbeats = conn.pipelined {
           procs.each do |key|
-            conn.hget(key, 'info')
+            conn.hget(key, "info")
           end
-        end
+        }
 
         # the hash named key has an expiry of 60 seconds.
         # if it's not found, that means the process has not reported
         # in to Redis and probably died.
-        to_prune = []
-        heartbeats.each_with_index do |beat, i|
-          to_prune << procs[i] if beat.nil?
-        end
-        count = conn.srem('processes', to_prune) unless to_prune.empty?
+        to_prune = procs.select.with_index { |proc, i|
+          heartbeats[i].nil?
+        }
+        count = conn.srem("processes", to_prune) unless to_prune.empty?
       end
       count
     end
 
     def each
-      procs = Sidekiq.redis { |conn| sscan(conn, 'processes') }.sort
+      result = Sidekiq.redis { |conn|
+        procs = conn.sscan_each("processes").to_a.sort
 
-      Sidekiq.redis do |conn|
         # We're making a tradeoff here between consuming more memory instead of
         # making more roundtrips to Redis, but if you have hundreds or thousands of workers,
         # you'll be happier this way
-        result = conn.pipelined do
+        conn.pipelined do
           procs.each do |key|
-            conn.hmget(key, 'info', 'busy', 'beat', 'quiet')
+            conn.hmget(key, "info", "busy", "beat", "quiet")
           end
         end
+      }
 
-        result.each do |info, busy, at_s, quiet|
-          # If a process is stopped between when we query Redis for `procs` and
-          # when we query for `result`, we will have an item in `result` that is
-          # composed of `nil` values.
-          next if info.nil?
+      result.each do |info, busy, at_s, quiet|
+        # If a process is stopped between when we query Redis for `procs` and
+        # when we query for `result`, we will have an item in `result` that is
+        # composed of `nil` values.
+        next if info.nil?
 
-          hash = Sidekiq.load_json(info)
-          yield Process.new(hash.merge('busy' => busy.to_i, 'beat' => at_s.to_f, 'quiet' => quiet))
-        end
+        hash = Sidekiq.load_json(info)
+        yield Process.new(hash.merge("busy" => busy.to_i, "beat" => at_s.to_f, "quiet" => quiet))
       end
-
-      nil
     end
 
     # This method is not guaranteed accurate since it does not prune the set
@@ -793,7 +812,7 @@ module Sidekiq
     # contains Sidekiq processes which have sent a heartbeat within the last
     # 60 seconds.
     def size
-      Sidekiq.redis { |conn| conn.scard('processes') }
+      Sidekiq.redis { |conn| conn.scard("processes") }
     end
 
     # Returns the identity of the current cluster leader or "" if no leader.
@@ -801,9 +820,9 @@ module Sidekiq
     # or Sidekiq Pro.
     def leader
       @leader ||= begin
-        x = Sidekiq.redis {|c| c.get("dear-leader") }
+        x = Sidekiq.redis { |c| c.get("dear-leader") }
         # need a non-falsy value so we can memoize
-        x = "" unless x
+        x ||= ""
         x
       end
     end
@@ -830,11 +849,11 @@ module Sidekiq
     end
 
     def tag
-      self['tag']
+      self["tag"]
     end
 
     def labels
-      Array(self['labels'])
+      Array(self["labels"])
     end
 
     def [](key)
@@ -842,23 +861,23 @@ module Sidekiq
     end
 
     def identity
-      self['identity']
+      self["identity"]
     end
 
     def quiet!
-      signal('TSTP')
+      signal("TSTP")
     end
 
     def stop!
-      signal('TERM')
+      signal("TERM")
     end
 
     def dump_threads
-      signal('TTIN')
+      signal("TTIN")
     end
 
     def stopping?
-      self['quiet'] == 'true'
+      self["quiet"] == "true"
     end
 
     private
@@ -872,7 +891,6 @@ module Sidekiq
         end
       end
     end
-
   end
 
   ##
@@ -897,19 +915,22 @@ module Sidekiq
   #
   class Workers
     include Enumerable
-    include RedisScanner
 
     def each
       Sidekiq.redis do |conn|
-        procs = sscan(conn, 'processes')
+        procs = conn.sscan_each("processes").to_a
         procs.sort.each do |key|
-          valid, workers = conn.pipelined do
+          valid, workers = conn.pipelined {
             conn.exists(key)
             conn.hgetall("#{key}:workers")
-          end
+          }
           next unless valid
           workers.each_pair do |tid, json|
-            yield key, tid, Sidekiq.load_json(json)
+            hsh = Sidekiq.load_json(json)
+            p = hsh["payload"]
+            # avoid breaking API, this is a side effect of the JSON optimization in #4316
+            hsh["payload"] = Sidekiq.load_json(p) if p.is_a?(String)
+            yield key, tid, hsh
           end
         end
       end
@@ -923,18 +944,17 @@ module Sidekiq
     # which can easily get out of sync with crashy processes.
     def size
       Sidekiq.redis do |conn|
-        procs = sscan(conn, 'processes')
+        procs = conn.sscan_each("processes").to_a
         if procs.empty?
           0
         else
-          conn.pipelined do
+          conn.pipelined {
             procs.each do |key|
-              conn.hget(key, 'busy')
+              conn.hget(key, "busy")
             end
-          end.map(&:to_i).inject(:+)
+          }.sum(&:to_i)
         end
       end
     end
   end
-
 end
