@@ -1,7 +1,8 @@
 # frozen_string_literal: true
-require 'socket'
-require 'securerandom'
-require 'sidekiq/exception_handler'
+
+require "socket"
+require "securerandom"
+require "sidekiq/exception_handler"
 
 module Sidekiq
   ##
@@ -10,18 +11,16 @@ module Sidekiq
   module Util
     include ExceptionHandler
 
-    EXPIRY = 60 * 60 * 24
-
     def watchdog(last_words)
       yield
     rescue Exception => ex
-      handle_exception(ex, { context: last_words })
+      handle_exception(ex, {context: last_words})
       raise ex
     end
 
     def safe_thread(name, &block)
       Thread.new do
-        Thread.current['sidekiq_label'] = name
+        Thread.current.name = name
         watchdog(name, &block)
       end
     end
@@ -34,8 +33,12 @@ module Sidekiq
       Sidekiq.redis(&block)
     end
 
+    def tid
+      Thread.current["sidekiq_tid"] ||= (Thread.current.object_id ^ ::Process.pid).to_s(36)
+    end
+
     def hostname
-      ENV['DYNO'] || Socket.gethostname
+      ENV["DYNO"] || Socket.gethostname
     end
 
     def process_nonce
@@ -43,22 +46,20 @@ module Sidekiq
     end
 
     def identity
-      @@identity ||= "#{hostname}:#{$$}:#{process_nonce}"
+      @@identity ||= "#{hostname}:#{::Process.pid}:#{process_nonce}"
     end
 
-    def fire_event(event, options={})
+    def fire_event(event, options = {})
       reverse = options[:reverse]
       reraise = options[:reraise]
 
       arr = Sidekiq.options[:lifecycle_events][event]
       arr.reverse! if reverse
       arr.each do |block|
-        begin
-          block.call
-        rescue => ex
-          handle_exception(ex, { context: "Exception during Sidekiq lifecycle event.", event: event })
-          raise ex if reraise
-        end
+        block.call
+      rescue => ex
+        handle_exception(ex, {context: "Exception during Sidekiq lifecycle event.", event: event})
+        raise ex if reraise
       end
       arr.clear
     end
